#!/usr/bin/env python
# -*- coding: koi8-r -*-
"""ALDC HTML writer"""
import metalexer
#from urllib import quote as __quote
Type=("html",("html","htm","HTML","HTM"),"ALDC HTML Writer",0.02, 0.03)
#quote=lambda s: __quote(s,"#.+=@$@!.&")
def quote(s):
  return "".join([ c in """/"'\t\n% """ and "_" or c for c in s ])


def F1_nbsp(*args):
  ret=args[2]["_"]
  return ret

def F1_intlink(*args):
  ret=args[2]["_"]%(quote(args[2]["linkid"]),args[2]["group"])
  return ret

def F1_m_k(*args):
  fn=args[0].genfootnotes()
  ret='%s'%(fn and "<hr>\n%s"%fn or '')
  return ret

def F1_verb(*args):
  ret=args[2]["_"]%(args[2]["vid"],args[2]["vbody"],args[2]["$"],args[2]["vtitle"])
  return ret

def F1_link(*args):
  ret=args[2]["_"]%(args[2]["url"],args[2]["group"] or ("<code>%s</code>"%args[2]["url"]))
  return ret

def F1_stitle(*args):
  ret=args[2]["_"]%(quote("ID.. "+args[1]),args[1])
  return ret

def F1_keyword(*args):
  ret=args[2]["_"]%(quote(args[2]["glossitem"]),args[2]["group"])
  return ret

def F1_footnote(*args):
  ret=args[2]["_"]%(args[2]["$"],args[2]["$"],args[2]["$"])
  return ret

def F1_ditem(*args):
  ret=args[2]["_"]%(args[2]["term"],args[2]["defbody"])
  return ret

def F1_id(*args):
  ret=args[2]["_"]%quote(args[1])
  return ret

def F1_object(*args):
  ret=args[2]["_"]%(args[2]["oid"],args[2]["opath"],args[2]["opath"],args[2]["$"],args[2]["otitle"])
  return ret

def F1_ptitle(*args):
  ret=args[2]["_"]%(quote("ID. "+args[1]),args[1])
  return ret

lexer={
"vid":	# verbatim ID
(
	"",
	"",
	"",
),
"primary":	# glossary primary term
(
	'',
	'<dt><strong>',
	'</strong></dt>',
),
"opath":	# object path
(
	'filetype',
	'',
	'',
),
"nbsp":	# punctuation: non-breakable space
(
	"format",
	"&nbsp;",
	F1_nbsp,
),
"secondaries":	# glossary secondary block
(
	'delete >>',
	'',
	'',
),
"intlink":	# in-text link
(
	'< > format',
	'<a href="%s">%s</a>',
	F1_intlink,
),
"deflist":	# definition list enclosure
(
	'',
	'\n<dl>',
	'</dl>\n',
),
"group":	# generic grouping
(
	'',
	'',
	'',
),
"quoted":	# quoted enclosure
(
	'< >',
	'&laquo;',
	'&raquo;',
),
"lllitem":	# list 3rd level item enclosure
(
	'',
	'<li>',
	'</li>\n',
),
"glossarticle":	# glossary definition itself
(
	'',
	'<dd>',
	'</dd>',
),
"system":	# system enclosure
(
	'< >',
	'<code>',
	'</code>',
),
"ndash":	# En-dash
(
	'',
	'',
	'&ndash;',
),
"dots":	# int-text three dots
(
	'>',
	'',
	'...',
),
"m_k":	# A document
(
	'< >',
	'',
	F1_m_k,
),
"defbody":	# definition list item body
(
	'',
	'',
	'',
),
"para":	# paragraph
(
	'',
	'\n<p>',
	'</p>\n',
),
"bid":	# bibliographical ID
(
	'delete',
	'',
	'',
),
"verb":	# verbatim enclosure
(
	'number format',
	'''
<span class="verbatim"><a name="ID%s"></a>
<hr>
<pre>%s</pre>
<hr><div align=center><strong> %d</strong>. %s</div><br>
</span>
''',
	F1_verb,
),
"link":	# link enclosure
(
	'< > format',
	'<a href="%s">%s</a>',
	F1_link,
),
"hl":	# in-text word highlighted part
(
	'quote=both',
	'<strong>',
	'</strong>',
),
"biblio":	# bibliographical link
(
	'< >',
	'[',
	']',
),
"litem":	# list 1st level item enclosure
(
	'',
	'<li>',
	'</li>\n',
),
"secondary":	# glossary secondary term
(
	'',
	'<strong><em>',
	'</em></strong>',
),
"info":	# marginale enclosure
(
	'',
	'<div class="marginale"><hr>',
	'<hr></div>',
),
"stitle":	# section title
(
	'picktitle format',
	'<a name="%s"></a><h2>%s</h2>\n',
	F1_stitle,
),
"term":	# definition list term
(
	"",
	"",
	"",
),
"word":	# in-text word enclosure
(
	'< >',
	'',
	'',
),
"keyword":	# keyword glossary enclosure
(
	'format < >',
	'<a href="glossary.html#%s">%s</a>',
	F1_keyword,
),
"linkid":	# link ID
(
	'',
	'#ID',
	'',
),
"list":	# list 1st level enclosure
(
	'',
	'\n<ul>',
	'</ul>\n',
),
"llist":	# list 2nd level enclosure
(
	'',
	'\n<ul>',
	'</ul>\n',
),
"error":	# red mark enclosure
(
	'>>',
	'<font color="red">',
	'</font>',
),
"qsystem":	# system quoted enclosure
(
	'< >',
	'&laquo;<code>',
	'</code>&raquo;',
),
"llitem":	# list 2nd level item enclosure
(
	'',
	'<li>',
	'</li>\n',
),
"sstitle":	# subsection title
(
	'picktitle',
	'<h3>',
	'</h3>\n',
),
"nnnitem":	# enumeration 3rd level item enclosure
(
	'',
	'<li>',
	'</li>\n',
),
"vbody":	# verbatim body
(
	"quote=pre",
	"",
	"",
),
"footnote":	# footnote enclosure
(
	'format number >>',
	'<a class="footnoteanc" name=".footanchor%d" href="#.footnote%d"><sup>%d</sup></a>',
	F1_footnote,
),
"otitle":	# object title
(
	"",
	"",
	"",
),
"emph":	# in-text emphasys enclosure
(
	'< >',
	'<em>',
	'</em>',
),
"eeenum":	# enumeration 3rd level enclosure
(
	'',
	'\n<ol>',
	'</ol>\n',
),
"ditem":	# definition list item enclosure
(
	'format',
	'<dt>%s</dt><dd>%s</dd>\n',
	F1_ditem,
),
"lllist":	# list 3rd level enclosure
(
	'',
	'\n<ul>',
	'</ul>\n',
),
"id":	# in-text ID
(
	'>> format',
	'<a name="ID%s"></a>',
	F1_id,
),
"repl":	# in-text replaceable
(
	'< >',
	'<samp><em>',
	'</em></samp>',
),
"glossdef":	# glossary definition enclosure
(
	'',
	'<div class="glossdef"><hr><dl>',
	'</dl><hr></div>',
),
"lpunct":	# punctuation sign with left space
(
	"< ",
	"",
	"",
),
"npunct":	# punctuation sign without spaces
(
	"",
	"",
	"",
),
"eenum":	# enumeration 2nd level enclosure
(
	'',
	'\n<ol>',
	'</ol>\n',
),
"glossitem":	# keyword glossary item
(
	'',
	'',
	'',
),
"sword":	# system word
(
	'',
	'',
	'',
),
"vtitle":	# verbatim title
(
	"",
	"",
	"",
),
"enum":	# enumeration 1st level enclosure
(
	'',
	'\n<ol>',
	'</ol>\n',
),
"normal":	# in-text word normal part
(
	'quote=both',
	'',
	'',
),
"oid":	# object ID
(
	"",
	"",
	"",
),
"punct":	# punctuation sign
(
	'>',
	'',
	'',
),
"nnitem":	# enumeration 2nd level item enclosure
(
	'',
	'<li>',
	'</li>\n',
),
"strong":	# strong enclosure
(
	'< >',
	'<strong>',
	'</strong>',
),
"ssect":	# subsection enclosure
(
	'',
	'\n',
	'\n',
),
"url":	# URL
(
	'',
	'',
	'',
),
"mdash":	# Em-dash
(
	'< > nbsp',
	'',
	'&mdash;',
),
"object":	# object enclosure
(
	'number format',
	'''
<p align=center><span class="illustration"><a name="ID%s"></a>
<img src="%s" alt="%s" border=0 style="width: 80%%"><br>
<strong> %d.</strong>%s</span></p>
''',
	F1_object,
),
"part":	# part enclosure
(
	'',
	'\n',
	'\n',
),
"ptitle":	# part title
(
	'picktitle format',
	'<a name="%s"></a><h1>%s</h1>\n',
	F1_ptitle,
),
"sect":	# section enclosure
(
	'',
	'\n',
	'\n',
),
"srepl":	# system replaceable
(
	'',
	'<em>',
	'</em>',
),
"nitem":	# enumeration 1st level item enclosure
(
	'',
	'<li>',
	'</li>\n',
),
}


class Writer(metalexer.Writer):
  def __init__(self, lexer):
    """metalexer.Writer.__init__
    Replace nbsp"""
    __=metalexer.tpg.re.compile
    metalexer.Writer.__init__(self,lexer)
    self.nbsp="&nbsp;"
    self._pattern=".body."	# default to HTML fragment
    self._quote={ "plain":	((__(r"'(.)"),r"<big>\1</big>"),
				),
    		  "pre":	(("&","&amp;"),
		      		 ("<","&lt;"),
		      		 (">","&gt;"),
				),
		}
    self._quote["both"]=self._quote["pre"]+self._quote["plain"]

  def pattern(self, title, divider):
    """Document to include generated text in
    first, replace the title"""
    patt=self._pattern.replace(".work.",title).replace("%","%%").replace(".body.","%s")
    return patt
  
  def genfootnotes(self):
    """Generate footnotes"""
    ret=[]
    for n in range(len(self.storage.get("footnote",[]))):
      ret.append('<div><p class="footnote"><a class="footnotesym" name=".footnote%d" href="#.footanchor%d">%d</a>%s</p></div>\n'%(n+1,n+1,n+1,self.storage["footnote"][n]))
    return "\n".join(ret)

  #formats={"dia": "png", "tex": "png", "ppm.bz2": "png", "ppm": "png", "*": "png" }
  formats={"png": "png", "jpg":"jpg", "jpeg":"jpeg", "gif":"gif", "*": "png" }

if __name__ == "__main__":
  import sys,mk_read
  R_mk=mk_read.Reader()
  result = R_mk(file(sys.argv[1]).read())
  writer = Writer(lexer)
  print writer(result)


