#!/usr/bin/env python
# -*- coding: koi8-r -*-
"""ALDC laTeX writer
 
----------

>	\\section
>>	\\subsection
>>>	\\subsubsection


-----------

#	%


-------
 
....................

.	\\begin{itemize}
	\\item{}
	\\end{itemize}
..	, 
...


...................

0	\\begin{enumerate}
	\\item{}
	\\end{enumerate}
00
000


.................

|| \\begin{description}
	\\item[]
	\\end{description}


--------------

# 
mdash	---
ndash	--

~  	~

_..._			\\emph{...}
_A_nything		\\Emph{A}nything

# 
``	<<
''	>>

<...>			\\keyword{...}
<...|>			\\Keyword{...}
<...|,,,>		\\Keyword[...]{,,,}
[...]			\\marginalia{...}
[...|]			\\Marginalia{...}
[...|,,,]		\\Marginalia[,,,]{...}
^...^			\\sys{...}
"..."			\\sys*{...}
`...'			\\sys*{...}
{...}			\\replace{...}
\\			
[[...]]			\\footnote{...}
@@...@@			\\label{...}
@...@			\\Ref{...}
(?... )		\\Error{...}



-----------

#     ,     
# LateX,     .eps
@@ Figure @ filename.dia @ label @@	\\Figure{Figure}{filename.eps}{label}

#    LaTeXe:
\\begin{Figure}{Caption}{label}
...   ...
\\end{Figure}
#  ,  :      .tex
#     ,         
#    . .


-------

@@ Example @ label @@	\\begin{Example}{Example}{label}
some text		some text
@@			\\end{Example}

"""

import metalexer
Type=("latex",("tex","TEX","TeX","latex","LATEX","laTeX"),"ALDC laTeX Writer",0.01,0.03)
ToProtect=['otitle','vtitle','ptitle','stitle','sstitle']


def F1_intlink(*args):
  ret=args[2]["_"]%(args[2]["group"],args[2]["linkid"])
  return ret

def F0_system(*args):
  ret=(set(ToProtect)&set(args[0].context) and '\\protect' or '')+'\\Sys{'
  return ret

def F1_verb(*args):
  ret=args[2]["_"]%(args[2]["vtitle"],args[2]["vid"],args[2]["vbody"]+(not args[2]["vbody"].endswith("\n") and "\n" or ""))
  return ret

def F1_link(*args):
  ret=args[2]["group"] and (args[2]["_"]%(args[2]["group"],args[2]["url"])) or ('\\Href{%s}'%args[2]["url"])
  return ret

def F1_biblio(*args):
  ret=args[2]["_"]%args[2]["bid"]
  return ret

def F1_stitle(*args):
  ret=args[2]["_"]%(args[1],"..@"+args[1].replace(" ","@"))
  return ret

def F1_keyword(*args):
  ret=args[2]["_"]%(args[2]["glossitem"],args[2]["group"])
  return ret

def F1_linkid(*args):
  ret=args[1].replace(" ","@")
  return ret

def F0_qsystem(*args):
  ret=(set(ToProtect)&set(args[0].context) and '\\protect' or '')+'\\Qsys{'
  return ret

def F1_ditem(*args):
  ret=args[2]["_"]%(args[2]["term"],args[2]["defbody"])
  return ret

def F1_glossdef(*args):
  ret=args[2]["_"]%(args[2]["glossarticle"],args[2]["primary"])
  return ret

def F1_object(*args):
  ret=args[2]["_"]%(args[2]["otitle"],args[2]["opath"],args[2]["oid"])
  return ret

def F1_ptitle(*args):
  ret=args[2]["_"]%(args[1],".@"+args[1].replace(" ","@"))
  return ret

lexer={
"vid":	# verbatim ID
(
	"",
	"",
	"",
),
"primary":	# glossary primary term
(
	'',
	'',
	'',
),
"opath":	# object path
(
	'filetype',
	'',
	'',
),
"nbsp":	# punctuation: non-breakable space
(
	"",
	"",
	'',
),
"secondaries":	# glossary secondary block
(
	'delete >>',
	'',
	'',
),
"intlink":	# in-text link
(
	'< > format',
	'\\Ref[%s]{%s}',
	F1_intlink,
),
"deflist":	# definition list enclosure
(
	'< > CR',
	'\\begin{description}\n',
	'\n\\end{description}',
),
"group":	# generic groupong
(
	'',
	'',
	'',
),
"quoted":	# quoted enclosure
(
	'< >',
	'<<',
	'>>',
),
"lllitem":	# list 3rd level item enclosure
(
	'< > CR',
	'\\item ',
	'',
),
"glossarticle":	# glossary definition itself
(
	'',
	'',
	'',
),
"system":	# system enclosure
(
	'< >',
	F0_system,
	'}',
),
"ndash":	# En-dash
(
	'',
	'',
	'"---',
),
"dots":	# int-text three dots
(
	'>',
	'',
	'\\dots',
),
"m_k":	# A document
(
	'< >',
	'',
	'',
),
"defbody":	# definition list item body
(
	'',
	'',
	'',
),
"para":	# paragraph
(
	'',
	'',
	'\n\n',
),
"bid":	# bibliographical ID
(
	'',
	'',
	'',
),
"verb":	# verbatim enclosure
(
	'number format < > CR',
	'''
\\begin{Example}{%s}{%s}
%s\\end{Example}''',
	F1_verb,
),
"link":	# link enclosure
(
	'< > format',
	'\\Href[%s]{%s}',
	F1_link,
),
"hl":	# in-text word highlighted part
(
	'quote',
	'\\Emph{',
	'}',
),
"biblio":	# bibliographical link
(
	'format',
	'[%s]',
	F1_biblio,
),
"litem":	# list 1st level item enclosure
(
	'< > CR',
	'\\item ',
	'',
),
"secondary":	# glossary secondary term
(
	'',
	'',
	'',
),
"info":	# marginale enclosure
(
	'< > CR',
	'\\marginalia{',
	'}',
),
"stitle":	# section title
(
	'picktitle format',
	'{%s}\n\\label{%s}\n',
	F1_stitle,
),
"term":	# definition list term
(
	"",
	"",
	"",
),
"word":	# in-text word enclosure
(
	'< >',
	'',
	'',
),
"keyword":	# keyword glossary enclosure
(
	'format < >',
	'\\Keyword[%s]{%s}',
	F1_keyword,
),
"linkid":	# link ID
(
	'format',
	'',
	F1_linkid,
),
"list":	# list 1st level enclosure
(
	'< > CR',
	'\\begin{itemize}\n',
	'\n\\end{itemize}',
),
"llist":	# list 2nd level enclosure
(
	'< > CR',
	'\\begin{itemize}\n',
	'\n\\end{itemize}',
),
"error":	# red mark enclosure
(
	'>>',
	'\\Error{',
	'}',
),
"qsystem":	# system quoted enclosure
(
	'< >',
	F0_qsystem,
	'}',
),
"llitem":	# list 2nd level item enclosure
(
	'< > CR',
	'\\item ',
	'',
),
"sstitle":	# subsection title
(
	'picktitle',
	'{',
	'}\n',
),
"nnnitem":	# enumeration 3rd level item enclosure
(
	'< > CR',
	'\\item ',
	'',
),
"vbody":	# verbatim body
(
	"",
	"",
	"",
),
"footnote":	# footnote enclosure
(
	'',
	'\\footnote{',
	'}',
),
"otitle":	# object title
(
	"",
	"",
	"",
),
"emph":	# in-text emphasys enclosure
(
	'< >',
	'\\emph{',
	'}',
),
"eeenum":	# enumeration 3rd level enclosure
(
	'< > CR',
	'\\begin{enumerate}\n',
	'\n\\end{enumerate}',
),
"ditem":	# definition list item enclosure
(
	'format < > CR',
	'\\item[%s] %s',
	F1_ditem,
),
"lllist":	# list 3rd level enclosure
(
	'< > CR',
	'\\begin{itemize}\n',
	'\n\\end{itemize}',
),
"id":	# in-text ID
(
	'>>',
	'\\label{',
	'}',
),
"repl":	# in-text replaceable
(
	'< >',
	'\\replace{',
	'}',
),
"glossdef":	# glossary definition enclosure
(
	'format < > CR',
	'\\Marginalia[%s]{%s}',
	F1_glossdef,
),
"lpunct":	# punctuation sign with left space
(
	"< ",
	"",
	"",
),
"npunct":	# punctuation sign without spaces
(
	"",
	"",
	"",
),
"eenum":	# enumeration 2nd level enclosure
(
	'< > CR',
	'\\begin{enumerate}\n',
	'\n\\end{enumerate}',
),
"glossitem":	# keyword glossary item
(
	'',
	'',
	'',
),
"sword":	# system word
(
	'quote',
	'',
	'',
),
"vtitle":	# verbatim title
(
	"",
	"",
	"",
),
"enum":	# enumeration 1st level enclosure
(
	'< > CR',
	'\\begin{enumerate}\n',
	'\n\\end{enumerate}',
),
"normal":	# in-text word normal part
(
	'quote',
	'',
	'',
),
"oid":	# object ID
(
	"",
	"",
	"",
),
"punct":	# punctuation sign
(
	'>',
	'',
	'',
),
"nnitem":	# enumeration 2nd level item enclosure
(
	'< > CR',
	'\\item ',
	'',
),
"strong":	# strong enclosure
(
	'< >',
	'\\keyword{',
	'}',
),
"ssect":	# subsection enclosure
(
	'< CR',
	'\\subsubsection',
	'',
),
"url":	# URL
(
	'',
	'',
	'',
),
"mdash":	# Em-dash
(
	'< >',
	'',
	'"---',
),
"object":	# object enclosure
(
	'number format < CR',
	'\\Figure{%s}{%s}{%s}\n',
	F1_object,
),
"part":	# part enclosure
(
	'< CR',
	'\\section',
	'',
),
"ptitle":	# part title
(
	'picktitle format',
	'{%s}\n\\label{%s}\n',
	F1_ptitle,
),
"sect":	# section enclosure
(
	'< CR',
	'\\subsection',
	'',
),
"srepl":	# system replaceable
(
	'quote',
	'\\replace{',
	'}',
),
"nitem":	# enumeration 1st level item enclosure
(
	'< > CR',
	'\\item ',
	'',
),
}


class Writer(metalexer.Writer):
  def __init__(self, lexer):
    """metalexer.Writer.__init__"""
    __=metalexer.tpg.re.compile
    metalexer.Writer.__init__(self,lexer)
    self.nbsp="~"
    self._pattern=".body."	# default to laTeX fragment
    self._quote=((__(r"([#$&_%{}\\])"),r"\\\1"),
		 ("^","\\textasciicircum{}"),
		 ("~","\\textasciitilde{}"),
		 (r"\\","\\textbackslash{}"),
		 ("'","\\'"),
		)

  def pattern(self, title, divider):
    """Document to include generated text in
    first, replace the title"""
    patt=self._pattern.replace(".work.",title).replace("%","%%").replace(".body.","%s")
    return patt

  formats={"latex":"latex", "*": "eps" }

if __name__ == "__main__":
  import sys,mk_read
  R_mk=mk_read.Reader()
  result = R_mk(file(sys.argv[1]).read())
  writer = Writer(lexer)
  print writer(result)


